# frozen_string_literal: true

require '/data_importer/lib/importers/github_repo'
require '/data_importer/lib/json_helper'

class GithubAdvisoryImporter < GithubRepo
  # repo has years that begin with 2017 as first GHSA
  YEAR_RANGE = (2017..Date.today.year).freeze

  EXPECTED_KEYS = %i[
    schema_version
    ghsa_id
    modified
    published
    aliases
    summary
    details
    severity
    affected
    references
    database_specific
  ].freeze

  EMPTY_HASH = EXPECTED_KEYS.map { |k| [k, nil] }.to_h.freeze

  def initialize
    super(repo_url = 'https://github.com/github/advisory-database.git', repo_path = '/data_importer/data/github_advisories')
  end

  def advisory_paths
    advisory_path = "#{repo_path}/advisories"
    {
      base_path: advisory_path,
      github_reviewed_path: "#{advisory_path}/github-reviewed",
      unreviewed_path: "#{advisory_path}/unreviewed"
    }
  end

  def list_jsons_for_year(year)
    json_wildcard = '*.json'
    github_reviewed_year_fp = "#{advisory_paths[:github_reviewed_path]}/#{year}/*/*"
    unreviewed_year_fp = "#{advisory_paths[:unreviewed_path]}/#{year}/*/*"

    github_reviewed_jsons_fp = Dir["#{github_reviewed_year_fp}/#{json_wildcard}"]
    unreviewed_jsons_fp = Dir["#{unreviewed_year_fp}/#{json_wildcard}"]

    {
      github_reviewed_jsons: github_reviewed_jsons_fp,
      unreviewed_jsons: unreviewed_jsons_fp
    }
  end

  def read_jsons_for_year(year)
    fp_hash = list_jsons_for_year(year)
    fns = fp_hash[:github_reviewed_jsons] + fp_hash[:unreviewed_jsons]
    jsons = fns.map do |fn|
      json = read_json(fn)
      attrs_from_item(json)
    end
    jsons.flatten
  end

  def attrs_from_item(json)
    attrs = JsonHelper.deep_transform_keys(json)
    attrs[:ghsa_id] = json[:id]
    attrs
  end

  def import
    pull_or_clone
    puts 'Now importing GithubAdvisories.'
    YEAR_RANGE.each do |year|
      puts "Importing advisory data from #{year}"
      jsons = read_jsons_for_year(year)
      hashes = jsons.map { |h| h.slice(*EXPECTED_KEYS).reverse_merge(EMPTY_HASH) }
      GithubAdvisory.upsert_all(hashes, unique_by: :ghsa_id)
    end
  end
end
